@@ -44,11 +44,6 @@
 #include <glib/gi18n-lib.h>
 #include <math.h>
 
-/* This number is fairly arbitrary. Long enough to show a pretty long
- * menu title, but not so long to make a menu grotesquely wide.
- */
-#define MAXIMUM_MENU_TITLE_LENGTH	48
-
 /* Used for window position & size sanity-checking. The sizes are big enough to prevent
  * at least normal-sized mate panels from obscuring the window at the screen edges.
  */
@@ -162,10 +157,7 @@ eel_gtk_window_set_initial_geometry (GtkWindow *window,
                                      guint width,
                                      guint height)
 {
-    GdkScreen *screen;
     int real_left, real_top;
-    int screen_width, screen_height;
-    int scale;
 
     g_return_if_fail (GTK_IS_WINDOW (window));
 
@@ -177,6 +169,10 @@ eel_gtk_window_set_initial_geometry (GtkWindow *window,
 
     if ((geometry_flags & EEL_GDK_X_VALUE) && (geometry_flags & EEL_GDK_Y_VALUE))
     {
+        GdkScreen *screen;
+        int screen_width, screen_height;
+        int scale;
+
         real_left = left;
         real_top = top;
 
@@ -448,6 +444,7 @@ GtkWidget *
 eel_image_menu_item_new_from_icon (const gchar *icon_name,
                                    const gchar *label_name)
 {
+    gchar *concat;
     GtkWidget *icon;
     GSettings *icon_settings;
     GtkWidget *box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
@@ -460,34 +457,38 @@ eel_image_menu_item_new_from_icon (const gchar *icon_name,
         /*Load an empty icon to hold the space*/
         icon = gtk_image_new ();
 
-    GtkWidget *label_menu = gtk_label_new_with_mnemonic (g_strconcat (label_name, "     ", NULL));
+    concat = g_strconcat (label_name, "     ", NULL);
+    GtkWidget *label_menu = gtk_label_new_with_mnemonic (concat);
     GtkWidget *menuitem = gtk_menu_item_new ();
 
     gtk_container_add (GTK_CONTAINER (box), icon);
 
     gtk_container_add (GTK_CONTAINER (box), label_menu);
- 
+
     gtk_container_add (GTK_CONTAINER (menuitem), box);
     gtk_widget_show_all (menuitem);
 
     g_object_unref(icon_settings);
+    g_free (concat);
 
     return menuitem;
 }
 
 GtkWidget *
-eel_image_menu_item_new_from_pixbuf (GdkPixbuf   *icon_pixbuf,
-                                     const gchar *label_name)
+eel_image_menu_item_new_from_surface (cairo_surface_t *icon_surface,
+                                      const gchar     *label_name)
 {
+    gchar *concat;
     GtkWidget *icon;
     GtkWidget *box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
 
-    if (icon_pixbuf)
-        icon = gtk_image_new_from_pixbuf (icon_pixbuf);
+    if (icon_surface)
+        icon = gtk_image_new_from_surface (icon_surface);
     else
         icon = gtk_image_new ();
 
-    GtkWidget *label_menu = gtk_label_new (g_strconcat (label_name, "     ", NULL));
+    concat = g_strconcat (label_name, "     ", NULL);
+    GtkWidget *label_menu = gtk_label_new (concat);
     GtkWidget *menuitem = gtk_menu_item_new ();
 
     gtk_container_add (GTK_CONTAINER (box), icon);
@@ -496,5 +497,65 @@ eel_image_menu_item_new_from_pixbuf (GdkPixbuf   *icon_pixbuf,
     gtk_container_add (GTK_CONTAINER (menuitem), box);
     gtk_widget_show_all (menuitem);
 
+    g_free (concat);
+
     return menuitem;
 }
+
+gboolean
+eel_notebook_scroll_event_cb (GtkWidget       *widget,
+                              GdkEventScroll  *event)
+{
+    GtkNotebook *notebook = GTK_NOTEBOOK (widget);
+    GtkWidget *child, *event_widget, *action_widget;
+
+    child = gtk_notebook_get_nth_page (notebook, gtk_notebook_get_current_page (notebook));
+    if (child == NULL)
+        return FALSE;
+
+    event_widget = gtk_get_event_widget ((GdkEvent*) event);
+
+    /* Ignore scroll events from the content of the page */
+    if (event_widget == NULL || event_widget == child || gtk_widget_is_ancestor (event_widget, child))
+        return FALSE;
+
+    /* And also from the action widgets */
+    action_widget = gtk_notebook_get_action_widget (notebook, GTK_PACK_START);
+    if (event_widget == action_widget || (action_widget != NULL && gtk_widget_is_ancestor (event_widget, action_widget)))
+        return FALSE;
+
+    action_widget = gtk_notebook_get_action_widget (notebook, GTK_PACK_END);
+    if (event_widget == action_widget || (action_widget != NULL && gtk_widget_is_ancestor (event_widget, action_widget)))
+        return FALSE;
+
+    switch (event->direction) {
+        case GDK_SCROLL_RIGHT:
+        case GDK_SCROLL_DOWN:
+            gtk_notebook_next_page (notebook);
+            break;
+        case GDK_SCROLL_LEFT:
+        case GDK_SCROLL_UP:
+            gtk_notebook_prev_page (notebook);
+            break;
+        case GDK_SCROLL_SMOOTH:
+            switch (gtk_notebook_get_tab_pos (notebook)) {
+                case GTK_POS_LEFT:
+                case GTK_POS_RIGHT:
+                    if (event->delta_y > 0)
+                        gtk_notebook_next_page (notebook);
+                    else if (event->delta_y < 0)
+                        gtk_notebook_prev_page (notebook);
+                    break;
+                case GTK_POS_TOP:
+                case GTK_POS_BOTTOM:
+                    if (event->delta_x > 0)
+                        gtk_notebook_next_page (notebook);
+                    else if (event->delta_x < 0)
+                        gtk_notebook_prev_page (notebook);
+                    break;
+            }
+            break;
+    }
+
+    return TRUE;
+}
