@@ -25,16 +25,17 @@
 */
 
 #include <config.h>
-#include "caja-desktop-directory.h"
+#include <gtk/gtk.h>
 
+#include <eel/eel-glib-extensions.h>
+#include <eel/eel-gtk-macros.h>
+
+#include "caja-desktop-directory.h"
 #include "caja-directory-private.h"
 #include "caja-file.h"
 #include "caja-file-private.h"
 #include "caja-file-utilities.h"
 #include "caja-global-preferences.h"
-#include <eel/eel-glib-extensions.h>
-#include <eel/eel-gtk-macros.h>
-#include <gtk/gtk.h>
 
 struct CajaDesktopDirectoryDetails
 {
@@ -162,6 +163,10 @@ directory_ready_callback (CajaDirectory *directory,
     g_assert (callback_data != NULL);
 
     merged_callback = callback_data;
+    /*Prevent segfaults on the assert with GTK 3.23*/
+    if (merged_callback->non_ready_directories == NULL)
+        return;
+
     g_assert (g_list_find (merged_callback->non_ready_directories, directory) != NULL);
 
     /* Update based on this call. */
