@@ -64,6 +64,7 @@ typedef enum
     CAJA_FILE_SORT_BY_SIZE,
     CAJA_FILE_SORT_BY_TYPE,
     CAJA_FILE_SORT_BY_MTIME,
+    CAJA_FILE_SORT_BY_BTIME,
     CAJA_FILE_SORT_BY_ATIME,
     CAJA_FILE_SORT_BY_EMBLEMS,
     CAJA_FILE_SORT_BY_TRASHED_TIME,
@@ -511,6 +512,7 @@ struct CajaFile
 typedef enum
 {
     CAJA_DATE_TYPE_MODIFIED,
+    CAJA_DATE_TYPE_CREATED,
     CAJA_DATE_TYPE_CHANGED,
     CAJA_DATE_TYPE_ACCESSED,
     CAJA_DATE_TYPE_PERMISSIONS_CHANGED,
