@@ -26,18 +26,19 @@
 */
 
 #include <config.h>
-#include "caja-open-with-dialog.h"
-#include "caja-signaller.h"
-
-#include <eel/eel-stock-dialogs.h>
-
 #include <string.h>
+#include <cairo-gobject.h>
+
 #include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
 #include <gio/gio.h>
 
+#include <eel/eel-stock-dialogs.h>
+
+#include "caja-open-with-dialog.h"
+#include "caja-signaller.h"
+
 #define sure_string(s)                    ((const char *)((s)!=NULL?(s):""))
-#define DESKTOP_ENTRY_GROUP		  "Desktop Entry"
 
 struct _CajaOpenWithDialogDetails
 {
@@ -219,12 +220,9 @@ static GAppInfo *
 add_or_find_application (CajaOpenWithDialog *dialog)
 {
     GAppInfo *app;
-    char *app_name;
-    const char *commandline;
     GError *error;
     gboolean success, should_set_default;
     char *message;
-    GList *applications;
 
     error = NULL;
     app = NULL;
@@ -234,6 +232,9 @@ add_or_find_application (CajaOpenWithDialog *dialog)
     }
     else
     {
+        char *app_name;
+        const char *commandline;
+
         commandline = gtk_entry_get_text (GTK_ENTRY (dialog->details->entry));
         app_name = get_app_name (commandline, &error);
         if (app_name != NULL)
@@ -248,18 +249,20 @@ add_or_find_application (CajaOpenWithDialog *dialog)
 
     if (app == NULL)
     {
-        message = g_strdup_printf (_("Could not add application to the application database: %s"), error->message);
+        message = g_strdup_printf (_("Could not add application to the application database: %s"), error ? error->message : _("Unknown error"));
         eel_show_error_dialog (_("Could not add application"),
                                message,
                                GTK_WINDOW (dialog));
         g_free (message);
-        g_error_free (error);
+
+        if (error)
+            g_error_free (error);
+
         return NULL;
     }
 
     should_set_default = (dialog->details->add_mode) ||
-                         (!dialog->details->add_mode &&
-                          gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->details->checkbox)));
+                          gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->details->checkbox));
     success = TRUE;
 
     if (should_set_default)
@@ -279,6 +282,8 @@ add_or_find_application (CajaOpenWithDialog *dialog)
     }
     else
     {
+        GList *applications;
+
         applications = g_app_info_get_all_for_type (dialog->details->content_type);
         if (dialog->details->content_type && applications != NULL)
         {
@@ -322,13 +327,13 @@ response_cb (CajaOpenWithDialog *dialog,
              int response_id,
              gpointer data)
 {
-    GAppInfo *application;
-
     switch (response_id)
     {
     case RESPONSE_OPEN:
         if (check_application (dialog))
         {
+            GAppInfo *application;
+
             application = add_or_find_application (dialog);
 
             if (application)
@@ -447,12 +452,12 @@ browse_clicked_cb (GtkWidget *button,
 
     dialog = CAJA_OPEN_WITH_DIALOG (user_data);
 
-    chooser = gtk_file_chooser_dialog_new (_("Select an Application"),
+    chooser = eel_file_chooser_dialog_new (_("Select an Application"),
                                            GTK_WINDOW (dialog),
                                            GTK_FILE_CHOOSER_ACTION_OPEN,
-                                           GTK_STOCK_CANCEL,
+                                           "process-stop",
                                            GTK_RESPONSE_CANCEL,
-                                           GTK_STOCK_OPEN,
+                                           "document-open",
                                            GTK_RESPONSE_OK,
                                            NULL);
     gtk_window_set_destroy_with_parent (GTK_WINDOW (chooser), TRUE);
@@ -490,32 +495,44 @@ entry_changed_cb (GtkWidget *entry,
     }
 }
 
-static GdkPixbuf *
-get_pixbuf_for_icon (GIcon *icon)
+#define CAJA_OPEN_WITH_DIALOG_ICON_SIZE 24
+static cairo_surface_t *
+get_surface_for_icon (GIcon *icon)
 {
-    GdkPixbuf  *pixbuf;
-    char *filename;
+    cairo_surface_t  *surface;
+    gint icon_scale;
+
+    surface = NULL;
+    icon_scale = gdk_window_get_scale_factor (gdk_get_default_root_window ());
 
-    pixbuf = NULL;
     if (G_IS_FILE_ICON (icon))
     {
+        char *filename;
+
         filename = g_file_get_path (g_file_icon_get_file (G_FILE_ICON (icon)));
         if (filename)
         {
-            pixbuf = gdk_pixbuf_new_from_file_at_size (filename, 24, 24, NULL);
+            GdkPixbuf *pixbuf;
+            pixbuf = gdk_pixbuf_new_from_file_at_size (filename,
+                                                       CAJA_OPEN_WITH_DIALOG_ICON_SIZE * icon_scale,
+                                                       CAJA_OPEN_WITH_DIALOG_ICON_SIZE * icon_scale,
+                                                       NULL);
+            surface = gdk_cairo_surface_create_from_pixbuf (pixbuf, icon_scale, NULL);
+            g_object_unref (pixbuf);
         }
         g_free (filename);
     }
     else if (G_IS_THEMED_ICON (icon))
     {
         const char * const *names;
-        char *icon_no_extension;
-        char *p;
 
         names = g_themed_icon_get_names (G_THEMED_ICON (icon));
 
         if (names != NULL && names[0] != NULL)
         {
+            char *icon_no_extension;
+            char *p;
+
             icon_no_extension = g_strdup (names[0]);
             p = strrchr (icon_no_extension, '.');
             if (p &&
@@ -525,22 +542,27 @@ get_pixbuf_for_icon (GIcon *icon)
             {
                 *p = 0;
             }
-            pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
-                                               icon_no_extension, 24, GTK_ICON_LOOKUP_FORCE_SIZE, NULL);
+            surface = gtk_icon_theme_load_surface (gtk_icon_theme_get_default (),
+                                                   icon_no_extension,
+                                                   CAJA_OPEN_WITH_DIALOG_ICON_SIZE,
+                                                   icon_scale,
+                                                   NULL,
+                                                   GTK_ICON_LOOKUP_FORCE_SIZE,
+                                                   NULL);
             g_free (icon_no_extension);
         }
     }
-    return pixbuf;
+    return surface;
 }
 
 static gboolean
 caja_open_with_dialog_add_icon_idle (CajaOpenWithDialog *dialog)
 {
-    GtkTreeIter   iter;
-    GtkTreePath  *path;
-    GdkPixbuf    *pixbuf;
-    GIcon *icon;
-    gboolean      long_operation;
+    cairo_surface_t *surface;
+    GIcon           *icon;
+    gboolean         long_operation;
+    GtkTreeIter      iter;
+    GtkTreePath     *path = NULL;
 
     long_operation = FALSE;
     do
@@ -573,12 +595,12 @@ caja_open_with_dialog_add_icon_idle (CajaOpenWithDialog *dialog)
             continue;
         }
 
-        pixbuf = get_pixbuf_for_icon (icon);
-        if (pixbuf)
+        surface = get_surface_for_icon (icon);
+        if (surface)
         {
             long_operation = TRUE;
-            gtk_list_store_set (dialog->details->program_list_store, &iter, COLUMN_ICON, pixbuf, -1);
-            g_object_unref (pixbuf);
+            gtk_list_store_set (dialog->details->program_list_store, &iter, COLUMN_ICON, surface, -1);
+            cairo_surface_destroy (surface);
         }
 
         /* don't go back into the main loop if this wasn't very hard to do */
@@ -596,14 +618,14 @@ caja_open_with_search_equal_func (GtkTreeModel *model,
                                   GtkTreeIter *iter,
                                   gpointer user_data)
 {
-    char *normalized_key;
-    char *name, *normalized_name;
-    char *path, *normalized_path;
-    char *basename, *normalized_basename;
+    char *name;
+    char *path;
     gboolean ret;
 
     if (key != NULL)
     {
+        char *normalized_key;
+
         normalized_key = g_utf8_casefold (key, -1);
         g_assert (normalized_key != NULL);
 
@@ -616,6 +638,8 @@ caja_open_with_search_equal_func (GtkTreeModel *model,
 
         if (name != NULL)
         {
+            char *normalized_name;
+
             normalized_name = g_utf8_casefold (name, -1);
             g_assert (normalized_name != NULL);
 
@@ -629,6 +653,9 @@ caja_open_with_search_equal_func (GtkTreeModel *model,
 
         if (ret && path != NULL)
         {
+            char *normalized_path;
+            char *basename, *normalized_basename;
+
             normalized_path = g_utf8_casefold (path, -1);
             g_assert (normalized_path != NULL);
 
@@ -675,7 +702,7 @@ caja_open_with_dialog_add_items_idle (CajaOpenWithDialog *dialog)
     /* create list store */
     dialog->details->program_list_store = gtk_list_store_new (NUM_COLUMNS,
                                           G_TYPE_APP_INFO,
-                                          GDK_TYPE_PIXBUF,
+                                          CAIRO_GOBJECT_TYPE_SURFACE,
                                           G_TYPE_ICON,
                                           G_TYPE_STRING,
                                           G_TYPE_STRING,
@@ -723,7 +750,7 @@ caja_open_with_dialog_add_items_idle (CajaOpenWithDialog *dialog)
     column = gtk_tree_view_column_new ();
     gtk_tree_view_column_pack_start (column, renderer, FALSE);
     gtk_tree_view_column_set_attributes (column, renderer,
-                                         "pixbuf", COLUMN_ICON,
+                                         "surface", COLUMN_ICON,
                                          NULL);
 
     renderer = gtk_cell_renderer_text_new ();
@@ -931,15 +958,18 @@ caja_open_with_dialog_init (CajaOpenWithDialog *dialog)
     gtk_widget_show (GTK_WIDGET (dialog->details->checkbox));
     gtk_box_pack_start (GTK_BOX (vbox), dialog->details->checkbox, FALSE, FALSE, 0);
 
-    gtk_dialog_add_button (GTK_DIALOG (dialog),
-                           GTK_STOCK_REMOVE,
+    eel_dialog_add_button (GTK_DIALOG (dialog),
+                           _("_Remove"),
+                           "list-remove",
                            RESPONSE_REMOVE);
+
     gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog),
                                        RESPONSE_REMOVE,
                                        FALSE);
 
-    gtk_dialog_add_button (GTK_DIALOG (dialog),
-                           GTK_STOCK_CANCEL,
+    eel_dialog_add_button (GTK_DIALOG (dialog),
+                           _("_Cancel"),
+                           "process-stop",
                            GTK_RESPONSE_CANCEL);
 
 
@@ -1005,7 +1035,6 @@ set_uri_and_type (CajaOpenWithDialog *dialog,
     char *label;
     char *emname;
     char *name, *extension;
-    char *description;
     char *checkbox_text;
 
     name = NULL;
@@ -1038,13 +1067,13 @@ set_uri_and_type (CajaOpenWithDialog *dialog,
             emname = g_strdup_printf ("<i>%s</i>", name);
             if (add_mode)
             {
-                /* first %s is a filename and second %s is a file extension */
+                /* Translators: first %s is a filename and second %s is a file extension */
                 label = g_strdup_printf (_("Open %s and other %s document with:"),
                                          emname, dialog->details->extension);
             }
             else
             {
-                /* the %s here is a file name */
+                /* Translators: the %s here is a file name */
                 label = g_strdup_printf (_("Open %s with:"), emname);
                 checkbox_text = g_strdup_printf (_("_Remember this application for %s documents"),
                                                  dialog->details->extension);
@@ -1064,6 +1093,8 @@ set_uri_and_type (CajaOpenWithDialog *dialog,
     }
     else
     {
+        char *description;
+
         dialog->details->content_type = g_strdup (mime_type);
         description = g_content_type_get_description (mime_type);
 
@@ -1077,16 +1108,16 @@ set_uri_and_type (CajaOpenWithDialog *dialog,
             emname = g_strdup_printf ("<i>%s</i>", name);
             if (add_mode)
             {
-                /* First %s is a filename, second is a description
+                /* Translators: First %s is a filename, second is a description
                  * of the type, eg "plain text document" */
                 label = g_strdup_printf (_("Open %s and other \"%s\" files with:"),
                                          emname, description);
             }
             else
             {
-                /* %s is a filename */
+                /* Translators: %s is a filename */
                 label = g_strdup_printf (_("Open %s with:"), emname);
-                /* %s is a file type description */
+                /* Translators: %s is a file type description */
                 checkbox_text = g_strdup_printf (_("_Remember this application for \"%s\" files"),
                                                  description);
 
