@@ -29,18 +29,20 @@
  */
 
 #include <config.h>
-#include "caja-tree-view-drag-dest.h"
+#include <stdio.h>
+#include <string.h>
+
+#include <gtk/gtk.h>
 
 #include <eel/eel-gtk-macros.h>
-#include <gtk/gtk.h>
+
+#include "caja-tree-view-drag-dest.h"
 #include "caja-file-dnd.h"
 #include "caja-file-changes-queue.h"
 #include "caja-icon-dnd.h"
 #include "caja-link.h"
 #include "caja-marshal.h"
 #include "caja-debug-log.h"
-#include <stdio.h>
-#include <string.h>
 
 #define AUTO_SCROLL_MARGIN 20
 
@@ -76,13 +78,11 @@ enum
     LAST_SIGNAL
 };
 
-static void caja_tree_view_drag_dest_init (CajaTreeViewDragDest      *dest);
-static void caja_tree_view_drag_dest_class_init    (CajaTreeViewDragDestClass *class);
-
 static guint signals[LAST_SIGNAL] = { 0 };
 
 G_DEFINE_TYPE (CajaTreeViewDragDest, caja_tree_view_drag_dest,
                G_TYPE_OBJECT);
+
 #define parent_class caja_tree_view_drag_dest_parent_class
 
 static const GtkTargetEntry drag_types [] =
@@ -333,7 +333,6 @@ static CajaFile *
 file_for_path (CajaTreeViewDragDest *dest, GtkTreePath *path)
 {
     CajaFile *file;
-    char *uri;
 
     if (path)
     {
@@ -341,6 +340,8 @@ file_for_path (CajaTreeViewDragDest *dest, GtkTreePath *path)
     }
     else
     {
+        char *uri;
+
         uri = get_root_uri (dest);
 
         file = NULL;
@@ -504,7 +505,6 @@ drag_motion_callback (GtkWidget *widget,
     CajaTreeViewDragDest *dest;
     GtkTreePath *path;
     GtkTreePath *drop_path, *old_drop_path;
-    GtkTreeModel *model;
     GtkTreeIter drop_iter;
     GtkTreeViewDropPosition pos;
     GdkWindow *bin_window;
@@ -547,6 +547,8 @@ drag_motion_callback (GtkWidget *widget,
 
     if (action)
     {
+        GtkTreeModel *model;
+
         set_drag_dest_row (dest, drop_path);
         model = gtk_tree_view_get_model (GTK_TREE_VIEW (widget));
         if (drop_path == NULL || (old_drop_path != NULL &&
@@ -905,8 +907,6 @@ drag_data_received_callback (GtkWidget *widget,
                              gpointer data)
 {
     CajaTreeViewDragDest *dest;
-    const char *tmp;
-    int length;
     gboolean success, finished;
 
     dest = CAJA_TREE_VIEW_DRAG_DEST (data);
@@ -947,11 +947,16 @@ drag_data_received_callback (GtkWidget *widget,
             success = TRUE;
             break;
         case CAJA_ICON_DND_RAW:
+        {
+            const char *tmp;
+            int length;
+
             length = gtk_selection_data_get_length (selection_data);
             tmp = gtk_selection_data_get_data (selection_data);
             receive_dropped_raw (dest, tmp, length, context, x, y);
             success = TRUE;
             break;
+        }
         case CAJA_ICON_DND_KEYWORD:
             receive_dropped_keyword (dest, context, x, y);
             success = TRUE;
@@ -1013,9 +1018,8 @@ set_direct_save_uri (CajaTreeViewDragDest *dest,
                      GdkDragContext *context,
                      int x, int y)
 {
-    GFile *base, *child;
     char *drop_uri;
-    char *filename, *uri;
+    char *uri;
 
     g_assert (dest->details->direct_save_uri == NULL);
 
@@ -1024,9 +1028,13 @@ set_direct_save_uri (CajaTreeViewDragDest *dest,
     drop_uri = get_drop_target_uri_at_pos (dest, x, y);
     if (drop_uri != NULL)
     {
+        char *filename;
+
         filename = get_direct_save_filename (context);
         if (filename != NULL)
         {
+            GFile *base, *child;
+
             /* Resolve relative path */
             base = g_file_new_for_uri (drop_uri);
             child = g_file_get_child (base, filename);
