@@ -76,6 +76,7 @@
 #include "caja-self-check-functions.h"
 #include "caja-notes-viewer.h"
 #include "caja-emblem-sidebar.h"
+#include "caja-bookmarks-sidebar.h"
 #include "caja-image-properties-page.h"
 #include "caja-desktop-window.h"
 #include "caja-spatial-window.h"
@@ -119,7 +120,7 @@ struct _CajaApplicationPrivate {
 
 G_DEFINE_TYPE_WITH_PRIVATE (CajaApplication, caja_application, GTK_TYPE_APPLICATION);
 
-GList *
+static GList *
 caja_application_get_spatial_window_list (void)
 {
     return caja_application_spatial_window_list;
@@ -210,7 +211,7 @@ caja_application_smclient_initialize (CajaApplication *self)
     /* TODO: Should connect to quit_requested and block logout on active transfer? */
 }
 
-void
+static void
 caja_application_smclient_startup (CajaApplication *self)
 {
     g_assert (self->smclient == NULL);
@@ -218,8 +219,9 @@ caja_application_smclient_startup (CajaApplication *self)
 }
 
 static void
-caja_empty_callback_to_ensure_read() {
+caja_empty_callback_to_ensure_read (CajaApplication *self) {
 /*do nothing, just exist to suppress runtime error*/
+    (void) self;
 }
 
 static void
@@ -366,13 +368,12 @@ caja_application_open (GApplication *app,
     gboolean browser_window = FALSE;
     gboolean open_in_tabs = FALSE;
     const gchar *geometry = NULL;
-    const char splitter = '=';
 
     g_debug ("Open called on the GApplication instance; %d files", n_files);
 
     /* Check if local command line passed --browser, --geometry or --tabs */
     if (strlen (options) > 0) {
-        gchar** splitedOptions = g_strsplit (options, &splitter, 3);
+        gchar** splitedOptions = g_strsplit (options, "=", 3);
         sscanf (splitedOptions[0], "%d", &browser_window);
         if (strcmp (splitedOptions[1], "NULL") != 0) {
             geometry = splitedOptions[1];
@@ -725,7 +726,7 @@ caja_application_create_desktop_windows (CajaApplication *application)
     }
 }
 
-void
+static void
 caja_application_open_desktop (CajaApplication *application)
 {
     if (caja_application_desktop_windows == NULL)
@@ -1388,12 +1389,12 @@ caja_application_get_session_data (CajaApplication *self)
     xmlSaveCtxtPtr ctx;
     xmlBufferPtr buffer;
 
-    doc = xmlNewDoc ("1.0");
+    doc = xmlNewDoc ((const xmlChar *) "1.0");
 
-    root_node = xmlNewNode (NULL, "session");
+    root_node = xmlNewNode (NULL, (const xmlChar *) "session");
     xmlDocSetRootElement (doc, root_node);
 
-    history_node = xmlNewChild (root_node, NULL, "history", NULL);
+    history_node = xmlNewChild (root_node, NULL, (const xmlChar *) "history", NULL);
 
     n_processed = 0;
     for (l = caja_get_history_list (); l != NULL; l = l->next) {
@@ -1404,10 +1405,10 @@ caja_application_get_session_data (CajaApplication *self)
 
         bookmark = l->data;
 
-        bookmark_node = xmlNewChild (history_node, NULL, "bookmark", NULL);
+        bookmark_node = xmlNewChild (history_node, NULL, (const xmlChar *) "bookmark", NULL);
 
         tmp = caja_bookmark_get_name (bookmark);
-        xmlNewProp (bookmark_node, "name", tmp);
+        xmlNewProp (bookmark_node, (const xmlChar *) "name", (const xmlChar *) tmp);
         g_free (tmp);
 
         icon = caja_bookmark_get_icon (bookmark);
@@ -1415,16 +1416,16 @@ caja_application_get_session_data (CajaApplication *self)
         tmp = g_icon_to_string (icon);
         g_object_unref (icon);
         if (tmp) {
-            xmlNewProp (bookmark_node, "icon", tmp);
+            xmlNewProp (bookmark_node, (const xmlChar *) "icon", (const xmlChar *) tmp);
             g_free (tmp);
         }
 
         tmp = caja_bookmark_get_uri (bookmark);
-        xmlNewProp (bookmark_node, "uri", tmp);
+        xmlNewProp (bookmark_node, (const xmlChar *) "uri", (const xmlChar *) tmp);
         g_free (tmp);
 
         if (caja_bookmark_get_has_custom_name (bookmark)) {
-            xmlNewProp (bookmark_node, "has_custom_name", "TRUE");
+            xmlNewProp (bookmark_node, (const xmlChar *) "has_custom_name", (const xmlChar *) "TRUE");
         }
 
         if (++n_processed > 50) { /* prevent history list from growing arbitrarily large. */
@@ -1455,49 +1456,49 @@ caja_application_get_session_data (CajaApplication *self)
             continue;
         }
 
-        win_node = xmlNewChild (root_node, NULL, "window", NULL);
+        win_node = xmlNewChild (root_node, NULL, (const xmlChar *) "window", NULL);
 
-        xmlNewProp (win_node, "location", tmp);
+        xmlNewProp (win_node, (const xmlChar *) "location", (const xmlChar *) tmp);
         g_free (tmp);
 
-        xmlNewProp (win_node, "type", CAJA_IS_NAVIGATION_WINDOW (window) ? "navigation" : "spatial");
+        xmlNewProp (win_node, (const xmlChar *) "type", CAJA_IS_NAVIGATION_WINDOW (window) ? (const xmlChar *) "navigation" : (const xmlChar *) "spatial");
 
         if (CAJA_IS_NAVIGATION_WINDOW (window)) { /* spatial windows store their state as file metadata */
             GdkWindow *gdk_window;
 
             tmp = eel_gtk_window_get_geometry_string (GTK_WINDOW (window));
-            xmlNewProp (win_node, "geometry", tmp);
+            xmlNewProp (win_node, (const xmlChar *) "geometry", (const xmlChar *) tmp);
             g_free (tmp);
 
             gdk_window = gtk_widget_get_window (GTK_WIDGET (window));
 
             if (gdk_window &&
                 gdk_window_get_state (gdk_window) & GDK_WINDOW_STATE_MAXIMIZED) {
-                xmlNewProp (win_node, "maximized", "TRUE");
+                xmlNewProp (win_node, (const xmlChar *) "maximized", (const xmlChar *) "TRUE");
             }
 
             if (gdk_window &&
                 gdk_window_get_state (gdk_window) & GDK_WINDOW_STATE_STICKY) {
-                xmlNewProp (win_node, "sticky", "TRUE");
+                xmlNewProp (win_node, (const xmlChar *) "sticky", (const xmlChar *) "TRUE");
             }
 
             if (gdk_window &&
                 gdk_window_get_state (gdk_window) & GDK_WINDOW_STATE_ABOVE) {
-                xmlNewProp (win_node, "keep-above", "TRUE");
+                xmlNewProp (win_node, (const xmlChar *) "keep-above", (const xmlChar *) "TRUE");
             }
         }
 
         for (m = slots; m != NULL; m = m->next) {
             slot = CAJA_WINDOW_SLOT (m->data);
 
-            slot_node = xmlNewChild (win_node, NULL, "slot", NULL);
+            slot_node = xmlNewChild (win_node, NULL, (const xmlChar *) "slot", NULL);
 
             tmp = caja_window_slot_get_location_uri (slot);
-            xmlNewProp (slot_node, "location", tmp);
+            xmlNewProp (slot_node, (const xmlChar *) "location", (const xmlChar *) tmp);
             g_free (tmp);
 
             if (slot == active_slot) {
-                xmlNewProp (slot_node, "active", "TRUE");
+                xmlNewProp (slot_node, (const xmlChar *) "active", (const xmlChar *) "TRUE");
             }
         }
 
@@ -1513,7 +1514,7 @@ caja_application_get_session_data (CajaApplication *self)
     }
 
     xmlSaveClose(ctx);
-    data = g_strndup (buffer->content, buffer->use);
+    data = g_strndup ((const char *) buffer->content, buffer->use);
     xmlBufferFree (buffer);
 
     xmlFreeDoc (doc);
@@ -1562,12 +1563,13 @@ caja_application_load_session (CajaApplication *application)
 
         for (node = root_node->children; node != NULL; node = node->next)
         {
-
-            if (g_strcmp0 (node->name, "text") == 0)
+            if (node->name == NULL || *node->name == '\0' ||
+                xmlStrcmp (node->name, (const xmlChar *) "text") == 0)
             {
                 continue;
             }
-            else if (g_strcmp0 (node->name, "history") == 0)
+
+            if (xmlStrcmp (node->name, (const xmlChar *) "history") == 0)
             {
                 xmlNodePtr bookmark_node;
                 gboolean emit_change;
@@ -1576,29 +1578,31 @@ caja_application_load_session (CajaApplication *application)
 
                 for (bookmark_node = node->children; bookmark_node != NULL; bookmark_node = bookmark_node->next)
                 {
-                    if (g_strcmp0 (bookmark_node->name, "text") == 0)
+                    if (bookmark_node->name == NULL || *bookmark_node->name == '\0' ||
+                        xmlStrcmp (bookmark_node->name, (const xmlChar *) "text") == 0)
                     {
                         continue;
                     }
-                    else if (g_strcmp0 (bookmark_node->name, "bookmark") == 0)
+
+                    if (xmlStrcmp (bookmark_node->name, (const xmlChar *) "bookmark") == 0)
                     {
                         xmlChar *name, *icon_str, *uri;
                         gboolean has_custom_name;
                         GIcon *icon;
                         GFile *location;
 
-                        uri = xmlGetProp (bookmark_node, "uri");
-                        name = xmlGetProp (bookmark_node, "name");
-                        has_custom_name = xmlHasProp (bookmark_node, "has_custom_name") ? TRUE : FALSE;
-                        icon_str = xmlGetProp (bookmark_node, "icon");
+                        uri = xmlGetProp (bookmark_node, (const xmlChar *) "uri");
+                        name = xmlGetProp (bookmark_node, (const xmlChar *) "name");
+                        has_custom_name = xmlHasProp (bookmark_node, (const xmlChar *) "has_custom_name") ? TRUE : FALSE;
+                        icon_str = xmlGetProp (bookmark_node, (const xmlChar *) "icon");
                         icon = NULL;
                         if (icon_str)
                         {
-                            icon = g_icon_new_for_string (icon_str, NULL);
+                            icon = g_icon_new_for_string ((const char *) icon_str, NULL);
                         }
-                        location = g_file_new_for_uri (uri);
+                        location = g_file_new_for_uri ((const char *) uri);
 
-                        emit_change |= caja_add_to_history_list_no_notify (location, name, has_custom_name, icon);
+                        emit_change |= caja_add_to_history_list_no_notify (location, (const char *) name, has_custom_name, icon);
 
                         g_object_unref (location);
 
@@ -1624,7 +1628,7 @@ caja_application_load_session (CajaApplication *application)
                 }
             }
 
-            else if (g_strcmp0 (node->name, "window") == 0)
+            else if (xmlStrcmp (node->name, (const xmlChar *) "window") == 0)
 
             {
                 CajaWindow *window;
@@ -1632,7 +1636,7 @@ caja_application_load_session (CajaApplication *application)
                 xmlNodePtr slot_node;
                 GFile *location;
 
-                type = xmlGetProp (node, "type");
+                type = xmlGetProp (node, (const xmlChar *) "type");
                 if (type == NULL)
                 {
                     g_message ("empty type node while parsing session data");
@@ -1640,7 +1644,7 @@ caja_application_load_session (CajaApplication *application)
                     continue;
                 }
 
-                location_uri = xmlGetProp (node, "location");
+                location_uri = xmlGetProp (node, (const xmlChar *) "location");
                 if (location_uri == NULL)
                 {
                     g_message ("empty location node while parsing session data");
@@ -1649,25 +1653,24 @@ caja_application_load_session (CajaApplication *application)
                     continue;
                 }
 
-                if (g_strcmp0 (type, "navigation") == 0)
+                if (xmlStrcmp (type, (const xmlChar *) "navigation") == 0)
                 {
                     xmlChar *geometry;
                     int i;
 
                     window = caja_application_create_navigation_window (application, gdk_screen_get_default ());
-                    geometry = xmlGetProp (node, "geometry");
-                    if (geometry != NULL)
+                    geometry = xmlGetProp (node, (const xmlChar *) "geometry");
+                    if (geometry != NULL && *geometry != '\0')
                     {
-                        eel_gtk_window_set_initial_geometry_from_string
-                        (GTK_WINDOW (window),
-                         geometry,
-                         CAJA_NAVIGATION_WINDOW_MIN_WIDTH,
-                         CAJA_NAVIGATION_WINDOW_MIN_HEIGHT,
-                         FALSE);
+                        eel_gtk_window_set_initial_geometry_from_string (GTK_WINDOW (window),
+                                                                         (const char *) geometry,
+                                                                         CAJA_NAVIGATION_WINDOW_MIN_WIDTH,
+                                                                         CAJA_NAVIGATION_WINDOW_MIN_HEIGHT,
+                                                                         FALSE);
                     }
                     xmlFree (geometry);
 
-                    if (xmlHasProp (node, "maximized"))
+                    if (xmlHasProp (node, (const xmlChar *) "maximized"))
                     {
                         gtk_window_maximize (GTK_WINDOW (window));
                     }
@@ -1676,7 +1679,7 @@ caja_application_load_session (CajaApplication *application)
                         gtk_window_unmaximize (GTK_WINDOW (window));
                     }
 
-                    if (xmlHasProp (node, "sticky"))
+                    if (xmlHasProp (node, (const xmlChar *) "sticky"))
                     {
                         gtk_window_stick (GTK_WINDOW (window));
                     }
@@ -1685,7 +1688,7 @@ caja_application_load_session (CajaApplication *application)
                         gtk_window_unstick (GTK_WINDOW (window));
                     }
 
-                    if (xmlHasProp (node, "keep-above"))
+                    if (xmlHasProp (node, (const xmlChar *) "keep-above"))
                     {
                         gtk_window_set_keep_above (GTK_WINDOW (window), TRUE);
                     }
@@ -1696,11 +1699,11 @@ caja_application_load_session (CajaApplication *application)
 
                     for (i = 0, slot_node = node->children; slot_node != NULL; slot_node = slot_node->next)
                     {
-                        if (g_strcmp0 (slot_node->name, "slot") == 0)
+                        if (slot_node->name && xmlStrcmp (slot_node->name, (const xmlChar *) "slot") == 0)
                         {
                             xmlChar *slot_uri;
 
-                            slot_uri = xmlGetProp (slot_node, "location");
+                            slot_uri = xmlGetProp (slot_node, (const xmlChar *) "location");
                             if (slot_uri != NULL)
                             {
                                 CajaWindowSlot *slot;
@@ -1714,10 +1717,10 @@ caja_application_load_session (CajaApplication *application)
                                     slot = caja_window_open_slot (window->details->active_pane, CAJA_WINDOW_OPEN_SLOT_APPEND);
                                 }
 
-                                location = g_file_new_for_uri (slot_uri);
+                                location = g_file_new_for_uri ((const char *) slot_uri);
                                 caja_window_slot_open_location (slot, location, FALSE);
 
-                                if (xmlHasProp (slot_node, "active"))
+                                if (xmlHasProp (slot_node, (const xmlChar *) "active"))
                                 {
                                     caja_window_set_active_slot (slot->pane->window, slot);
                                 }
@@ -1731,20 +1734,18 @@ caja_application_load_session (CajaApplication *application)
                     if (i == 0)
                     {
                         /* This may be an old session file */
-                        location = g_file_new_for_uri (location_uri);
+                        location = g_file_new_for_uri ((const char *) location_uri);
                         caja_window_slot_open_location (window->details->active_pane->active_slot, location, FALSE);
                         g_object_unref (location);
                     }
                 }
-                else if (g_strcmp0 (type, "spatial") == 0)
+                else if (xmlStrcmp (type, (const xmlChar *) "spatial") == 0)
                 {
-                    location = g_file_new_for_uri (location_uri);
+                    location = g_file_new_for_uri ((const char *) location_uri);
                     window = caja_application_get_spatial_window (application, NULL, NULL,
-                    											  location, gdk_screen_get_default (),
-                    											  NULL);
-
-					caja_window_go_to (window, location);
-
+                                                                  location, gdk_screen_get_default (),
+                                                                  NULL);
+                    caja_window_go_to (window, location);
                     g_object_unref (location);
                 }
                 else
@@ -1783,6 +1784,7 @@ do_cmdline_sanity_checks (CajaApplication *self,
               gboolean perform_self_check,
               gboolean version,
               gboolean kill_shell,
+              gboolean select_uris,
               gchar **remaining)
 {
     gboolean retval = FALSE;
@@ -1806,6 +1808,12 @@ do_cmdline_sanity_checks (CajaApplication *self,
         goto out;
     }
 
+    if (select_uris && remaining == NULL) {
+        g_printerr ("%s\n",
+                _("--select must be used with at least an URI."));
+        goto out;
+    }
+
     retval = TRUE;
 
  out:
@@ -1830,6 +1838,55 @@ do_perform_self_checks (gint *exit_status)
     *exit_status = EXIT_SUCCESS;
 }
 
+static void
+select_items_ready_cb (GObject *source,
+                       GAsyncResult *res,
+                       gpointer user_data)
+{
+    GDBusConnection *connection = G_DBUS_CONNECTION (source);
+    CajaApplication *self = user_data;
+    GError *error = NULL;
+
+    g_dbus_connection_call_finish (connection, res, &error);
+
+    if (error != NULL) {
+        g_warning ("Unable to select specified URIs %s\n", error->message);
+        g_error_free (error);
+
+        /* open default location instead */
+        g_application_open (G_APPLICATION (self), NULL, 0, "");
+    }
+}
+
+static void
+caja_application_select (CajaApplication *self,
+                         GFile **files,
+                         gint len)
+{
+    GDBusConnection *connection = g_application_get_dbus_connection (G_APPLICATION (self));
+    GVariantBuilder builder;
+    gint idx;
+    gchar *uri;
+
+    g_variant_builder_init (&builder, G_VARIANT_TYPE ("as"));
+    for (idx = 0; idx < len; idx++) {
+        uri = g_file_get_uri (files[idx]);
+        g_variant_builder_add (&builder, "s", uri);
+        g_free (uri);
+    }
+
+    g_dbus_connection_call (connection,
+                            CAJA_FDO_DBUS_NAME,
+                            CAJA_FDO_DBUS_PATH,
+                            CAJA_FDO_DBUS_IFACE,
+                            "ShowItems",
+                            g_variant_new ("(ass)", &builder, ""), NULL,
+                            G_DBUS_CALL_FLAGS_NONE, G_MAXINT, NULL,
+                            select_items_ready_cb, self);
+
+    g_variant_builder_clear (&builder);
+}
+
 static gboolean
 running_in_mate (void)
 {
@@ -1856,6 +1913,7 @@ caja_application_local_command_line (GApplication *application,
     gboolean kill_shell = FALSE;
     const gchar *autostart_id;
     gboolean no_default_window = FALSE;
+    gboolean select_uris = FALSE;
     gchar **remaining = NULL;
     CajaApplication *self = CAJA_APPLICATION (application);
 
@@ -1884,6 +1942,8 @@ caja_application_local_command_line (GApplication *application,
           N_("Open a browser window."), NULL },
         { "quit", 'q', 0, G_OPTION_ARG_NONE, &kill_shell,
           N_("Quit Caja."), NULL },
+        { "select", 's', 0, G_OPTION_ARG_NONE, &select_uris,
+          N_("Select specified URI in parent folder."), NULL },
         { G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_STRING_ARRAY, &remaining, NULL,  N_("[URI...]") },
 
         { NULL }
@@ -1929,7 +1989,7 @@ caja_application_local_command_line (GApplication *application,
     }
 
     if (!do_cmdline_sanity_checks (self, perform_self_check,
-                       version, kill_shell, remaining)) {
+                                   version, kill_shell, select_uris, remaining)) {
         *exit_status = EXIT_FAILURE;
         goto out;
     }
@@ -1992,7 +2052,7 @@ caja_application_local_command_line (GApplication *application,
         g_strfreev (remaining);
     }
 
-    if (files == NULL && !no_default_window) {
+    if (files == NULL && !no_default_window && !select_uris) {
         files = g_malloc0 (2 * sizeof (GFile *));
         len = 1;
 
@@ -2000,9 +2060,15 @@ caja_application_local_command_line (GApplication *application,
         files[1] = NULL;
     }
 
-    /*Set up --geometry, --browser and --tabs options  */
-    /*Invoke "Open" to create new windows */
-    if (len > 0)  {
+    if (len == 0) {
+        goto out;
+    }
+
+    if (select_uris) {
+        caja_application_select (self, files, len);
+    } else {
+        /*Set up --geometry, --browser and --tabs options  */
+        /*Invoke "Open" to create new windows */
         gchar* concatOptions = g_malloc0(64);
         if (self->priv->geometry == NULL) {
             g_snprintf (concatOptions, 64, "%d=NULL=%d", browser_window, open_in_tabs);
@@ -2212,6 +2278,7 @@ caja_application_startup (GApplication *app)
     caja_history_sidebar_register ();
     caja_notes_viewer_register (); /* also property page */
     caja_emblem_sidebar_register ();
+    caja_bookmarks_sidebar_register ();
 
     /* register property pages */
     caja_image_properties_page_register ();
@@ -2227,7 +2294,7 @@ caja_application_startup (GApplication *app)
     menu_provider_init_callback ();
 
     /* Initialize notifications for eject operations */
-    notify_init (GETTEXT_PACKAGE);
+    notify_init (PACKAGE);
 
     /* Watch for unmounts so we can close open windows */
     /* TODO-gio: This should be using the UNMOUNTED feature of GFileMonitor instead */
