@@ -30,10 +30,12 @@
 #ifndef CAJA_LOCATION_BAR_H
 #define CAJA_LOCATION_BAR_H
 
+#include <gtk/gtk.h>
+
+#include <libcaja-private/caja-entry.h>
+
 #include "caja-navigation-window.h"
 #include "caja-navigation-window-pane.h"
-#include <libcaja-private/caja-entry.h>
-#include <gtk/gtk.h>
 
 #define CAJA_TYPE_LOCATION_BAR caja_location_bar_get_type()
 #define CAJA_LOCATION_BAR(obj) \
@@ -47,12 +49,12 @@
 #define CAJA_LOCATION_BAR_GET_CLASS(obj) \
   (G_TYPE_INSTANCE_GET_CLASS ((obj), CAJA_TYPE_LOCATION_BAR, CajaLocationBarClass))
 
-typedef struct CajaLocationBarDetails CajaLocationBarDetails;
+typedef struct _CajaLocationBarPrivate CajaLocationBarPrivate;
 
 typedef struct CajaLocationBar
 {
     GtkHBox parent;
-    CajaLocationBarDetails *details;
+    CajaLocationBarPrivate *details;
 } CajaLocationBar;
 
 typedef struct
